/*
 * Copyright (c) 2005
 *    Christian Biere <christianbiere@gmx.de> All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the authors nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef CONNECTION_HEADER_FILE
#define CONNECTION_HEADER_FILE

#include "common.h"
#include "net_addr.h"

typedef enum {
  CONN_EV_NONE    = 0,
  CONN_EV_READ    = 1,
  CONN_EV_WRITE   = 2,
  CONN_EV_RDWR    = CONN_EV_READ | CONN_EV_WRITE,
  CONN_EV_HANGUP  = 4,
  CONN_EV_ERROR   = 8,
  CONN_EV_TIMEOUT = 16,

  CONN_EV_ANY     = 31
} connection_event_t;

typedef struct connection connection_t;
typedef void (*connection_event_cb)(connection_t *, connection_event_t);

connection_t * connection_new(int, const net_addr_t addr, uint16_t, time_t now);
connection_t * connection_listen(const net_addr_t addr, uint16_t, int);
connection_t * connection_accept(connection_t *) NON_NULL;
connection_t * connection_udp(const net_addr_t addr, uint16_t port);

struct connection {
  net_addr_t  (* get_addr)      (connection_t *) NON_NULL;
  uint16_t  (* get_port)      (connection_t *) NON_NULL;
  int       (* get_fd)        (connection_t *) NON_NULL;
  time_t    (* get_stamp)     (connection_t *) NON_NULL;
  char *    (* get_addrstr)   (connection_t *, char *, size_t) NON_NULL;
  void      (* event)    (connection_t *, connection_event_t, time_t) NON_NULL;
  void      (* set_event_cb)  (connection_t *, connection_event_cb);
  connection_event_t (* get_eventmask) (connection_t *) NON_NULL;
  void      (* set_eventmask) (connection_t *, connection_event_t);
  void *    (* get_context)   (connection_t *) NON_NULL;
  void      (* set_context)   (connection_t *, void *);
  unsigned int  (* get_maxidle)   (connection_t *) NON_NULL;
  void      (* set_maxidle)   (connection_t *, unsigned int) NON_NULL;
  int64_t   (* get_age)       (connection_t *) NON_NULL;
  int       (* set_rcvbuf)    (connection_t *, int) NON_NULL;
  int       (* set_sndbuf)    (connection_t *, int) NON_NULL;
  int       (* set_nodelay)   (connection_t *, bool); 
  int       (* set_tos_lowdelay)    (connection_t *) NON_NULL; 
  int       (* set_tos_throughput)  (connection_t *) NON_NULL; 
  int       (* set_tos_reliability) (connection_t *) NON_NULL; 
  int       (* set_blocking)  (connection_t *, bool block);
  int       (* set_rcvlowat)  (connection_t *, unsigned int) NON_NULL;
  bool      (* get_listening) (connection_t *) NON_NULL;
  void      (* close)         (connection_t *) NON_NULL;
  void      (* destruct)      (connection_t *) NON_NULL;
};


/* vi: set ai et sts=2 sw=2 cindent: */
#endif /* CONNECTION_HEADER_FILE */
